/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive.home

import android.util.Log
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.foundation.background
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.scale
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import kotlinx.coroutines.launch
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.ui.graphics.painter.Painter
import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.FontWeight
import com.bbc.denadrive.R
import com.bbc.denadrive.apihandlers.MyFile
import com.bbc.denadrive.apihandlers.ResponsesViewer

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ScaffoldWithSidebar(
    responsesViewer: ResponsesViewer,
    onFileClick: (MyFile) -> Unit,
    onFolderClick: (MyFile) -> Unit,
    doIt: () -> Unit,
) {
    // State to control the drawer
    val drawerState = rememberDrawerState(initialValue = DrawerValue.Closed)
    val scope = rememberCoroutineScope()

    // ModalNavigationDrawer
    ModalNavigationDrawer(
        drawerState = drawerState,
        drawerContent = {
            // Drawer content
            Column(
                modifier = Modifier
                    .fillMaxHeight()
                    .fillMaxWidth(0.67f)
                    .background(Color.White)
            ) {
                RectangularButton(backgroundColor = Color.White, text = "Text 1") { }
//                Spacer(modifier = Modifier.height(10.dp))
                RectangularButton(backgroundColor = Color.White, text = "Text 2") { }
//                Spacer(modifier = Modifier.height(10.dp))
                RectangularButton(backgroundColor = Color.White, text = "Text 3") { }
            }
        },
        content = {
            // Main content of the screen
            Scaffold(
                topBar = {
                    TopAppBar(
                        title = { Text("DeNa Drive") },
                        navigationIcon = {
                            IconButton(onClick = {
                                scope.launch {
                                    drawerState.open()
                                }
                            }) {
                                // Icon for the menu (three horizontal lines)
                                Icon(Icons.Filled.Menu, contentDescription = "Menu")
                            }
                        },
                        actions = {
                            IconButton(onClick = { doIt() }) {
                                // Icon for the settings (three vertical dots)
                                Icon(Icons.Filled.MoreVert, contentDescription = "Settings")
                            }
                        }
                    )
                },
                content = { innerPadding ->
                    Box(
                        modifier = Modifier
                            .fillMaxSize()
                            .padding(innerPadding)
                            .padding(16.dp),
                        contentAlignment = Alignment.Center,
                    ) {
                        AnotherVisualizer(
                            responsesViewer,
                            onFileClick,
                            onFolderClick
                        )
//                        MyListVisualizer(
//                            responsesViewer = responsesViewer,
//                            onFileClick = onFileClick,
//                            onFolderClick = onFolderClick
//                        )
                    }
                }
            )
        }
    )
}

@Composable
fun RectangularButton(
    backgroundColor: Color,
    text: String,
    onClick: () -> Unit
) {
    var isPressed by remember { mutableStateOf(false) }
    val scale by animateFloatAsState(if (isPressed) 0.8f else 1f)

    Box(
        modifier = Modifier
            .fillMaxWidth()
            .height(48.dp) // Set the height of the button
            .background(color = backgroundColor) // Set the background color
            .scale(scale) // Apply the scale effect
            .pointerInput(Unit) {
                // Detect press and release gestures
                detectTapGestures(
                    onPress = {
                        isPressed = true
                        // Wait for the click to be released
                        tryAwaitRelease()
                        isPressed = false
                    },
                    onTap = { onClick() } // Call the onClick action
                )
            }
            .padding(8.dp),
        contentAlignment = Alignment.Center // Center the text
    ) {
        Text(text = text, style = MaterialTheme.typography.bodyLarge) // Button text
    }
}

@Composable
fun MyListVisualizer(
    responsesViewer: ResponsesViewer,
    onFileClick: (String) -> Unit,
    onFolderClick: (String) -> Unit
) {
    val fileList = mutableListOf<MyFile>()
    if (responsesViewer.lastResponseBody.value == null) {
        fileList.clear()
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
            Text(text = "Loading...", style = MaterialTheme.typography.titleLarge)
        }
    } else {
        fileList.clear()
        fileList.addAll(
            responsesViewer.lastResponseBody.value!!.toList()
        )
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.TopCenter) {
            LazyColumn {
                items(fileList) { file ->
                    val logo = painterResource(id = getId(file.mimeType))
                    val isFolder = file.mimeType.endsWith("folder")
                    val onClick = if (isFolder) onFolderClick
                    else onFileClick
                    LogoTextBox(logo, file.name) {
                        onClick(
                            if (isFolder) file.id else file.name
                        )
                    }
                }
            }
        }
    }
}

@Composable
fun Loading(
    modifier: Modifier = Modifier
) {
    // State to control the rotation animation
    val rotation = remember { mutableFloatStateOf(0f) }

    // Animate the rotation
    LaunchedEffect(Unit) {
        while (true) {
            rotation.floatValue += 10f
            if (rotation.floatValue >= 360f) rotation.floatValue = 0f
            kotlinx.coroutines.delay(16) // Approximately 60 FPS
        }
    }

    Box(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp),
        contentAlignment = Alignment.Center
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.Center
        ) {
            // Rotating Circular Progress Indicator
            CircularProgressIndicator(
                modifier = Modifier
                    .size(64.dp)
                    .graphicsLayer(rotationZ = rotation.floatValue),
                color = Color.Blue
            )
            Spacer(modifier = Modifier.height(16.dp))
            // Loading text
            Text(
                text = "Loading",
                fontSize = 20.sp,
                fontWeight = FontWeight.Bold,
                color = Color.Black
            )
        }
    }
}

@Composable
fun AnotherVisualizer(
    responsesViewer: ResponsesViewer,
    onFileClick: (MyFile) -> Unit,
    onFolderClick: (MyFile) -> Unit
) {
    val currentFolderTree = responsesViewer.currentFolderTree.value
    val updater1 = responsesViewer.updater1.value
    val updater2 = responsesViewer.updater2.value
    if (currentFolderTree.isLeaf() && (updater1 || updater2)) {
        Log.e("Visualizer", "AnotherVisualizer: ${currentFolderTree.value}", )
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
            Text(text = "How did you get here? You're inside a file...")
        }
    } else if (!currentFolderTree.loaded) {
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
            Loading()
        }
    } else if (currentFolderTree.getChildren().isEmpty()) {
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
            Text("Empty folder")
//            Loading()
        }
    } else {
        val fileList = currentFolderTree.getChildren().map { tree -> tree.value }
//        Log.e("Visualizer", "AnotherVisualizer: ${currentFolderTree.value}", )
//        Log.e("Visualizer", "AnotherVisualizer: ${currentFolderTree.getChildren()}", )
        Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.TopCenter) {
            LazyColumn {
                items(fileList) { file ->
                    val logo = painterResource(id = getId(file.mimeType))
                    val isFolder = file.mimeType.endsWith("folder")
                    val onClick = if (isFolder) onFolderClick
                    else onFileClick
                    LogoTextBox(logo, file.name) {
                        onClick(file)
                    }
                }
            }
        }
    }
}

fun getId(mimeType: String): Int {
    val extracted = mimeType.substringAfterLast(".")
    return when (extracted) {
        "document" -> R.drawable.docslogo_ok
        "presentation" -> R.drawable.slideslogo_ok
        "folder" -> R.drawable.folderlogo_ok
        else -> R.drawable.nuovaicona_
    }
}

@Composable
fun LogoTextBox(
    logoPainter: Painter,
    text: String,
    onClick: () -> Unit
) {
    Card(
        modifier = Modifier
            .fillMaxWidth()
            .padding(4.dp)
            .clickable(onClick = onClick),
        shape = RoundedCornerShape(8.dp),
        elevation = CardDefaults.cardElevation(defaultElevation = 4.dp)
    ) {
        Row(
            modifier = Modifier
                .padding(horizontal = 16.dp)
                .fillMaxWidth(),
            verticalAlignment = Alignment.CenterVertically
        ) {
            // Logo on the left
            Image(
                painter = logoPainter,
                contentDescription = "Item Logo",
                contentScale = ContentScale.Crop,
                alignment = Alignment.Center,
                modifier = Modifier
                    .size(80.dp) // Adjust size as needed
                    .padding(horizontal = 16.dp) // Space between logo and text
            )

            // Text to the right of the logo
            Text(
                text = text,
                fontSize = 20.sp,
                color = Color.Black
            )
        }
    }
}
